/* The host side of the bcrypt implementation is based on hello_world.c
* Modified by Katja Malvoni to fit the bcrypt implementation
*/

/*
hello_world.c

Copyright (C) 2012 Adapteva, Inc.
Contributed by Yaniv Sapir <yaniv@adapteva.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program, see the file COPYING. If not, see
<http://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#include <e-hal.h>

#define _BufSize (72)
#define _BufOffset (0x01000000)

int main(int argc, char *argv[])
{
	unsigned row, col, coreid;
	e_platform_t platform;
	e_epiphany_t dev;
	e_mem_t emem;
	char emsg[_BufSize];
	struct timeval start, end;
	
	e_init(NULL);
	if(e_reset_system() == E_ERR)
	{
		fprintf(stderr, "Reset failed!\n");
		exit(1);
	}	
	e_get_platform_info(&platform);
	
	e_alloc(&emem, _BufOffset, _BufSize);

	row = platform.rows / 2 - 1;
	col = platform.cols / 2;
	coreid = (row + platform.row) * 64 + col + platform.col;
	fprintf(stderr, "Message from eCore 0x%03x (%2d,%2d): ", coreid, row, col); fflush(stderr);

	e_open(&dev, row, col, 1, 1);
	e_reset_core(&dev, 0, 0);
	
	gettimeofday(&start, NULL);	
	
	e_load("e_bcrypt.srec", &dev, 0, 0, e_true);
	
	// Wait for core program execution to finish
	while(!emsg[70])
		e_read(&emem, 0, 0, 0x0, emsg, _BufSize);
	
	gettimeofday(&end, NULL);

	fprintf(stderr, "%s\n", emsg);	
	fprintf(stderr, "Execution time - Epiphany: %f ms\n", (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec)/1000.0));

	e_close(&dev);
	e_free(&emem);
	e_finalize();

	return 0;
}

